/*
 * C program to compute the value of pi using Monte Carlo
 *
 * command-line arguments:  number_of_samples, seed
 *
 * OpenMP parallel version
 */
#include <stdio.h>
#include <stdlib.h>
#include <time.h> 
#include <math.h>   /* has M_PI, fabs() */
#include "omp.h"
#include "simple-generator.h"

/* main program */

int main(int argc, char* argv[]) {

    int num_samples;
    int seed;
    double start_time, end_time;
    int count = 0;
    double x, y;
    int i;
    double pi;
    int nthreads;

    /* process command-line arguments */
    if (argc != 3)  {
        fprintf(stderr, "usage:  %s number_of_samples seed\n", argv[0]);
        exit(EXIT_FAILURE);
    }
    num_samples = atoi(argv[1]);
    seed = atoi(argv[2]);
    if ((num_samples <= 0) || (seed <= 0)) {
        fprintf(stderr, "usage:  %s number_of_samples seed\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    /* record start time */
    clock_t t1=clock();
    start_time =(double) t1;

    /* do calculation */ 
#pragma omp parallel
    {
        unsigned int rand_work;
#pragma omp master
        {
            nthreads = omp_get_num_threads();
        }
        /* somewhat crude way to give every thread a different seed */
        rand_work = seed + omp_get_thread_num();
#pragma omp for private(x,y) reduction(+:count) schedule(static)
        for (i = 0; i < num_samples; ++i) {
            x = simple_random(&rand_work);
            y = simple_random(&rand_work);
            if ((x*x + y*y) <= 1.0)
                ++count;
        }
#pragma omp master
        {
            pi = 4.0 * (double) count / (double) num_samples;
        }
    }

    /* record end time and print results */
    t1=clock();
    end_time = (double) t1;
    printf("OpenMP C program results with %d threads:\n", nthreads);
    printf("number of samples = %d, seed = %d\n", num_samples, seed);
    printf("estimated pi = %12.10f\n", pi);
    printf("difference between estimated pi and math.h M_PI = %12.10f\n", 
            fabs(pi - M_PI));
    printf("time to compute = %g seconds\n", (end_time - start_time));

    /* clean up and return */
    return EXIT_SUCCESS;
}
